@@ -28,8 +28,9 @@
 //!   this code handles low-level equality and subtyping operations. The
 //!   type check pass in the compiler is found in the `librustc_typeck` crate.
 //!
-//! For a deeper explanation of how the compiler works and is
-//! organized, see the README.md file in this directory.
+//! For more information about how rustc works, see the [rustc guide].
+//!
+//! [rustc guide]: https://rust-lang-nursery.github.io/rustc-guide/
 //!
 //! # Note
 //!
@@ -42,27 +43,39 @@
 
 #![feature(box_patterns)]
 #![feature(box_syntax)]
-#![feature(conservative_impl_trait)]
+#![cfg_attr(stage0, feature(conservative_impl_trait))]
 #![feature(const_fn)]
+#![cfg_attr(stage0, feature(copy_closures, clone_closures))]
 #![feature(core_intrinsics)]
 #![feature(drain_filter)]
-#![feature(i128_type)]
-#![feature(match_default_bindings)]
-#![feature(inclusive_range_syntax)]
+#![feature(dyn_trait)]
+#![feature(entry_or_default)]
+#![feature(from_ref)]
+#![feature(fs_read_write)]
+#![cfg_attr(stage0, feature(i128_type, i128))]
+#![cfg_attr(stage0, feature(inclusive_range_syntax))]
 #![cfg_attr(windows, feature(libc))]
+#![cfg_attr(stage0, feature(match_default_bindings))]
+#![feature(macro_lifetime_matcher)]
 #![feature(macro_vis_matcher)]
 #![feature(never_type)]
+#![feature(exhaustive_patterns)]
+#![feature(non_exhaustive)]
 #![feature(nonzero)]
+#![feature(proc_macro_internals)]
 #![feature(quote)]
 #![feature(refcell_replace_swap)]
 #![feature(rustc_diagnostic_macros)]
 #![feature(slice_patterns)]
 #![feature(specialization)]
 #![feature(unboxed_closures)]
-#![feature(underscore_lifetimes)]
+#![cfg_attr(stage0, feature(underscore_lifetimes))]
+#![cfg_attr(stage0, feature(universal_impl_trait))]
 #![feature(trace_macros)]
+#![feature(trusted_len)]
+#![feature(catch_expr)]
 #![feature(test)]
-#![feature(const_atomic_bool_new)]
+#![feature(inclusive_range_fields)]
 
 #![recursion_limit="512"]
 
@@ -72,9 +85,9 @@ extern crate core;
 extern crate fmt_macros;
 extern crate getopts;
 extern crate graphviz;
+#[macro_use] extern crate lazy_static;
 #[cfg(windows)]
 extern crate libc;
-extern crate owning_ref;
 extern crate rustc_back;
 #[macro_use] extern crate rustc_data_structures;
 extern crate serialize;
@@ -84,9 +97,14 @@ extern crate rustc_errors as errors;
 #[macro_use] extern crate syntax;
 extern crate syntax_pos;
 extern crate jobserver;
+extern crate proc_macro;
 
 extern crate serialize as rustc_serialize; // used by deriving
 
+extern crate rustc_apfloat;
+extern crate byteorder;
+extern crate backtrace;
+
 // Note that librustc doesn't actually depend on these crates, see the note in
 // `Cargo.toml` for this crate about why these are here.
 #[allow(unused_extern_crates)]
@@ -130,7 +148,6 @@ pub mod middle {
     pub mod recursion_limit;
     pub mod resolve_lifetime;
     pub mod stability;
-    pub mod trans;
     pub mod weak_lang_items;
 }
 
@@ -140,6 +157,7 @@ pub mod traits;
 pub mod ty;
 
 pub mod util {
+    pub mod captures;
     pub mod common;
     pub mod ppaux;
     pub mod nodemap;
